/* -*- Mode: C -*- */

/*-------------------------------------------------------------------*\
 * File: rstruct.h
 * Author: Bryan Jurish
\*-------------------------------------------------------------------*/

/**
 * \file rstruct.h
 * \brief High-level structures & typedefs for librsynth TTS system.
 */

#ifndef _RSYNTH_RSTRUCT_H
#define _RSYNTH_RSTRUCT_H

/*-------------------------------------------------------------------
 * local global #define's
 *-------------------------------------------------------------------*/
/// RSYNTH_DEBUG: global debugging symbol
//#define RSYNTH_DEBUG
#undef RSYNTH_DEBUG

/// whether to warn verbosely by default
//#define RSYNTH_DICT_WARN_VERBOSE
#undef RSYNTH_DICT_WARN_VERBOSE

/*-------------------------------------------------------------------
 * typedefs & globals
 *-------------------------------------------------------------------*/
#include <rsynth/rtypes.h>
#include <rsynth/aufile.h>

//-- bit flags for rsynth_synth.flags
//#define RSY_USEAUDIO 1
/// bit flag for rsynth_synth.flags
#define RSY_RUNNING 1

/// enum for rsynth_synth.au_mode (unused?!)
typedef enum rsynth_au_mode_e {
  rsy_au_none, ///< no audio output
  rsy_au_dev,  ///< device output (?)
  rsy_au_sun,  ///< SuN/NeXT format
  rsy_au_raw,  ///< raw linear format
  rsy_au_float ///< float format
} rsynth_au_mode;

/*-------------------------------------------------------------------
 * new structures
 *-------------------------------------------------------------------*/

/// rsynth_synth: synthesizer-global data structure
typedef struct _rsynth_synth {
  //-- OUTPUT
  int flags;     ///< bit flags
  int verbose;   ///< verbosity level (currently only 0 or 1)

  /**
   * this gets set when an arg is '-help': really a
   * hack to allow us to keep using Nick's 'getarg.c'
   */
  int help_only; 

  //-- AUDIO: properties
  long samp_rate;          ///< sample rate (Hz)
  unsigned au_encoding;    ///< black magic
  unsigned au_size;        ///< also black magic
  //rsynth_au_mode au_mode;  // ... audio write-mode

  //-- AUDIO: fds
  //int dev_fd;            // fd to audio output device
  //int linear_fd;         // fd to raw-linear output file
  int au_fd;               ///< fd to audio output file

  //-- AUDIO: filenames
  //char *dev_filename;    // audio device
  //char *linear_filename; // raw linear output file
  char *au_filename;       ///< audio output filename

  //-- AUDIO: functions
  /**
   * Really a 'void (*file_write_p)'.
   * Should be callable to write audio file output.
   * use like: '(*(file_write_p)synth->file_write)(ARGS);'.
   */
  void (*file_write);
  /**
   * Really a 'void (*file_term_p)'.
   * Should be callable to finish audio file output.
   */
  void (*file_term);

  //-- SYNTH: Klatt guts
  klatt_frame_t def_pars;      ///< default values for Klatt synth parameters
  klatt_global_t klatt_global; ///< ye olde fynthefyzer guttf
  double mSec_per_frame;       ///< length of Klatt frames
  int impulse;                 ///< ?
  int casc;                    ///< (probably) number of Klatt cascades
  
  //-- HOLMES (phoneme-->vocal-tract-parameters guts [also seems to do some prosidy!])
  int speed;                   ///< Holmes speed coefficient
  char *par_name;              ///< parameter filename (get rid of!)
  FILE *par_file;              ///< parameter file output
  //char *jsru_name;
  //FILE *jsru_file;
  char *kpar_name;
  FILE *kpar_file;
  char *epar_name;
  FILE *epar_file;
  double frac;                 ///< Parameter filter 'fraction'

  //-- DICTIONARY
  rsynth_dict rsdict;          ///< Dictionary structure

  //-- PHONETICS (do we need this instance-specific?)
  //trie_ptr phtoelm;

} rsynth_synth, *rsynth_synth_ptr;


/*-------------------------------------------------------------------
 * new prototypes: struct manipulators
 *-------------------------------------------------------------------*/

/**
 * synth = rsynth_new();
 *   + create & initialize a new synth object with default values
 *   + uses rsynth_initialize() to init the new synth
 */
extern rsynth_synth *rsynth_new PROTO((void));

/**
 * rsynth_initialize(synth);
 *   + initialize a synth object with default values
 *   + returns boolean
 */
extern int rsynth_initialize PROTO((rsynth_synth *synth));


/**
 * rsynth_clear(synth);
 *   + "private" pre-destructor: frees substructure
 */
extern void rsynth_clear PROTO((rsynth_synth *synth));

/**
 * rsynth_destroy(synth);
 *   + destroy the synth: frees memory
 */
extern void rsynth_destroy PROTO((rsynth_synth *synth));



/**
 * rsynth_configure(synth,argc,argv);
 *     + configures synth according to options in argc,argv
 */
extern int rsynth_configure PROTO((rsynth_synth *synth, int argc, char **argv));

/**
 * rsynth_start(synth)
 *   + starts the synth, return is boolean
 */
extern int rsynth_start PROTO((rsynth_synth *synth));

/**
 * rsynth_stop(synth)
 *   + stops a running synth, return is boolean
 */
extern int rsynth_stop PROTO((rsynth_synth *synth));

/**
 * rsynth_sync(synth)
 *   + syncs a running synth, return is boolean
 */
extern int rsynth_stop PROTO((rsynth_synth *synth));


//-- audio
extern int rsynth_start_audio PROTO((rsynth_synth *synth));
extern int rsynth_stop_audio PROTO((rsynth_synth *synth));
extern int rsynth_sync_audio PROTO((rsynth_synth *synth));

//-- klatt synth
extern int rsynth_start_synth PROTO((rsynth_synth *synth));
extern int rsynth_stop_synth PROTO((rsynth_synth *synth));

//-- holmes
extern int rsynth_start_holmes PROTO((rsynth_synth *synth));
extern int rsynth_stop_holmes PROTO((rsynth_synth *synth));

//-- dict
extern int rsynth_start_dict PROTO((rsynth_synth *synth));
extern int rsynth_stop_dict PROTO((rsynth_synth *synth));

//-- file
extern int rsynth_start_file PROTO((rsynth_synth *synth));
extern int rsynth_stop_file PROTO((rsynth_synth *synth));
extern int rsynth_sync_file PROTO((rsynth_synth *synth));


#endif /* _RSYNTH_RSTRUCT_H */
